@@ -17,6 +17,7 @@
 
 from __future__ import absolute_import
 from __future__ import print_function
+from __future__ import unicode_literals
 
 import sys
 
@@ -25,6 +26,12 @@ from . import tokenize
 from . import utils
 
 
+try:
+    unicode
+except NameError:
+    unicode = str
+
+
 __author__ = 'nnorwitz@google.com (Neal Norwitz)'
 
 
@@ -92,7 +99,7 @@ class Node(object):
         return '%s(%d, %d, %s)' % (name, self.start, self.end, suffix)
 
     def __repr__(self):
-        return str(self)
+        return unicode(self)
 
 
 class Define(Node):
@@ -129,7 +136,8 @@ class Goto(Node):
         self.label = label
 
     def __str__(self):
-        return self._string_helper(self.__class__.__name__, str(self.label))
+        return self._string_helper(self.__class__.__name__,
+                                   unicode(self.label))
 
 
 class Expr(Node):
@@ -143,7 +151,7 @@ class Expr(Node):
         return False
 
     def __str__(self):
-        return self._string_helper(self.__class__.__name__, str(self.expr))
+        return self._string_helper(self.__class__.__name__, unicode(self.expr))
 
 
 class Return(Expr):
@@ -168,7 +176,8 @@ class Using(Node):
         self.names = names
 
     def __str__(self):
-        return self._string_helper(self.__class__.__name__, str(self.names))
+        return self._string_helper(self.__class__.__name__,
+                                   unicode(self.names))
 
 
 class Parameter(Node):
@@ -184,7 +193,7 @@ class Parameter(Node):
         return self.type.name == node.name
 
     def __str__(self):
-        name = str(self.type)
+        name = unicode(self.type)
         suffix = '%s %s' % (name, self.name)
         if self.default:
             suffix += ' = ' + ''.join([d.name for d in self.default])
@@ -344,10 +353,7 @@ class Function(_GenericDeclaration):
         return self.body is not None
 
     def is_exportable(self):
-        if (
-            (self.return_type and 'static' in self.return_type.modifiers) or
-            self.templated_types
-        ):
+        if self.return_type and 'static' in self.return_type.modifiers:
             return False
         return None not in self.namespace
 
@@ -406,7 +412,7 @@ class Type(_GenericDeclaration):
         prefix = ''
         if self.modifiers:
             prefix = ' '.join(self.modifiers) + ' '
-        name = str(self.name)
+        name = unicode(self.name)
         if self.templated_types:
             name += '<%s>' % self.templated_types
         suffix = prefix + name
@@ -946,7 +952,8 @@ class AstBuilder(object):
             tokens.append(next_token)
             # Handle templated names.
             if next_token.name == '<':
-                tokens.extend(self._get_matching_char('<', '>', get_next_token))
+                tokens.extend(self._get_matching_char('<', '>',
+                                                      get_next_token))
                 last_token_was_name = True
             next_token = get_next_token()
         return tokens, next_token
@@ -973,8 +980,15 @@ class AstBuilder(object):
             assert token.name == '(', token
 
         name = return_type_and_name.pop()
+        if (
+            len(return_type_and_name) > 2 and
+            return_type_and_name[-1].name == 'operator'
+        ):
+            op = return_type_and_name.pop()
+            name = tokenize.Token(tokenize.NAME, 'operator' + name.name,
+                                  op.start, name.end)
         # Handle templatized ctors.
-        if name.name == '>':
+        elif name.name == '>':
             index = 1
             while return_type_and_name[index].name != '<':
                 index += 1
@@ -1002,12 +1016,14 @@ class AstBuilder(object):
         if name.name == self.in_class and not modifiers:
             modifiers |= FUNCTION_CTOR
         parameters = list(self._get_parameters())
-        del parameters[-1]              # Remove trailing ')'.
+        last_token = parameters.pop()    # Remove trailing ')'.
 
         # Handling operator() is especially weird.
         if name.name == 'operator' and not parameters:
             token = self._get_next_token()
             assert token.name == '(', token
+            name = tokenize.Token(tokenize.NAME, 'operator()',
+                                  name.start, last_token.end)
             parameters = list(self._get_parameters())
             del parameters[-1]          # Remove trailing ')'.
 
@@ -1648,7 +1664,7 @@ class AstBuilder(object):
 
     def handle_goto(self):
         tokens = self._get_tokens_up_to(tokenize.SYNTAX, ';')
-        assert len(tokens) == 1, str(tokens)
+        assert len(tokens) == 1, unicode(tokens)
         return Goto(tokens[0].start, tokens[0].end, tokens[0].name)
 
     def handle_try(self):
