@@ -25,6 +25,7 @@ suppress warnings.
 
 from __future__ import absolute_import
 from __future__ import print_function
+from __future__ import unicode_literals
 
 import os
 import sys
@@ -35,6 +36,13 @@ from . import keywords
 from . import metrics
 from . import symbols
 from . import tokenize
+from . import utils
+
+
+try:
+    basestring
+except NameError:
+    basestring = str
 
 
 __author__ = 'nnorwitz@google.com (Neal Norwitz)'
@@ -96,7 +104,7 @@ class WarningHunter(object):
 
     def _add_warning(self, msg, node, filename=None):
         if filename is not None:
-            src_metrics = metrics.Metrics(open(filename).read())
+            src_metrics = metrics.Metrics(utils.read_file(filename))
         else:
             filename = self.filename
             src_metrics = self.metrics
@@ -176,8 +184,8 @@ class WarningHunter(object):
         return included_files, forward_declarations
 
     def _verify_includes(self, included_files):
-        # Read and parse all the #include'd files and warn about really
-        # stupid things that can be determined from the #include'd file name.
+        """Read and parse all the #include'd files and warn about really stupid
+        things that can be determined from the #include'd file name."""
         files_seen = {}
         for filename, (node, module) in included_files.items():
             normalized_filename = module.normalized_filename
@@ -195,7 +203,7 @@ class WarningHunter(object):
                 files_seen[normalized_filename] = node
 
     def _verify_include_files_used(self, file_uses, included_files):
-        # Find all #include files that are unnecessary.
+        """Find all #include files that are unnecessary."""
         for include_file, use in file_uses.items():
             if use != USES_DECLARATION:
                 node, module = included_files[include_file]
@@ -205,22 +213,22 @@ class WarningHunter(object):
                         msg += '. Use references instead'
                     self._add_warning(msg, node)
 
-    def _verify_forward_declarations_used(
-        self, forward_declarations, decl_uses,
-            file_uses):
-        # Find all the forward declarations that are not used.
+    def _verify_forward_declarations_used(self, forward_declarations,
+                                          decl_uses, file_uses):
+        """Find all the forward declarations that are not used."""
         for cls in forward_declarations:
             if decl_uses[cls] == UNUSED:
                 node = forward_declarations[cls]
                 if cls in file_uses:
-                    msg = ('%r forward declared, but needs to be #included' %
-                           cls)
+                    msg = (
+                        "'{}' forward declared, "
+                        'but needs to be #included'.format(cls))
                 else:
-                    msg = '%r not used' % cls
+                    msg = "'{}' not used".format(cls)
                 self._add_warning(msg, node)
 
     def _determine_uses(self, included_files, forward_declarations):
-        # Setup the use type of each symbol.
+        """Set up the use type of each symbol."""
         file_uses = dict.fromkeys(included_files, UNUSED)
         decl_uses = dict.fromkeys(forward_declarations, UNUSED)
         symbol_table = self.symbol_table
@@ -238,7 +246,7 @@ class WarningHunter(object):
             if isinstance(name, list):
                 # name contains a list of tokens.
                 name = '::'.join([n.name for n in name])
-            elif not isinstance(name, str):
+            elif not isinstance(name, basestring):
                 # Happens when variables are defined with inlined types, e.g.:
                 #   enum {...} variable;
                 return
@@ -394,7 +402,8 @@ class WarningHunter(object):
                 self._add_warning('%s not found %s' % (name, where), node)
 
     def _check_public_functions(self, primary_header, all_headers):
-        # Verify all the public functions are also declared in a header file.
+        """Verify all the public functions are also declared in a header
+        file."""
         public_symbols = ()
         declared_only_symbols = {}
         if primary_header:
@@ -435,10 +444,10 @@ class WarningHunter(object):
 
         for name, declared_only in declared_only_symbols.items():
             if declared_only:
-                # TODO(nnorwitz): shouldn't warn if function is templatized.
                 node = public_symbols[name]
-                msg = '%s declared but not defined' % name
-                self._add_warning(msg, node, primary_header.filename)
+                if not node.templated_types:
+                    msg = '%s declared but not defined' % name
+                    self._add_warning(msg, node, primary_header.filename)
 
     def _get_primary_header(self, included_files):
         basename = os.path.splitext(self.normalized_filename)[0]
