@@ -17,6 +17,7 @@
 
 from __future__ import absolute_import
 from __future__ import print_function
+from __future__ import unicode_literals
 
 import sys
 
@@ -27,13 +28,15 @@ __author__ = 'nnorwitz@google.com (Neal Norwitz)'
 
 # Add $ as a valid identifier char since so much code uses it.
 _letters = 'abcdefghijklmnopqrstuvwxyz'
-VALID_IDENTIFIER_CHARS = set(_letters + _letters.upper() + '_0123456789$')
-HEX_DIGITS = set('0123456789abcdefABCDEF')
-INT_OR_FLOAT_DIGITS = set('01234567890eE-+')
+VALID_IDENTIFIER_CHARS = frozenset(_letters +
+                                   _letters.upper() +
+                                   '_0123456789$')
+HEX_DIGITS = frozenset('0123456789abcdefABCDEF')
+INT_OR_FLOAT_DIGITS = frozenset('01234567890eE-+')
 
 
 # C++0x string preffixes.
-_STR_PREFIXES = set(('R', 'u8', 'u8R', 'u', 'uR', 'U', 'UR', 'L', 'LR'))
+_STR_PREFIXES = frozenset(('R', 'u8', 'u8R', 'u', 'uR', 'U', 'UR', 'L', 'LR'))
 
 
 # Token types.
